import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.*; 
import java.time.LocalDate; 
import java.time.format.DateTimeFormatter; 
import java.time.temporal.ChronoUnit; 
import peasy.*; 
import controlP5.*; 
import http.requests.*; 
import java.net.URLEncoder; 
import org.json.*; 
import org.json.JSONObject; 
import org.json.JSONArray; 

import org.json.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class trending_video extends PApplet {

/********************************************************
 * Data Visualization - Most popular items by year      *
 * Tested in Processing 3.5.3                           *
 *                                                      *
 * Author: Jiaheng Tang                                 *
 * Supervisor: George Legrady                           *
 * Adapted from Mert Toka's work in MAT 259 in 2016     *
 *                                                      *
 * Purpose: Visualizing the trending Youtube videos     *
 *          from 2017/11/14 to 2018/06/14               *
 *                                                      *
 * Usage:   Press toggle button on bottom left side     *
 *          to toggle records by category.              *
 *          Hover over records to show their title      *
 *          likes, dislikes, and views.                 *
 *          Click on the records to show their channel  *
 *          info. Right-click to cancel highlight       *
 *                                                      *
 ********************************************************/













final static int Z_DISTANCE = 2200;
final static float DIMENSION = 2000;
final static int HIGH_ALPHA = 255;
final static int NORMAL_ALPHA = 100;
final static int LOW_ALPHA = 10;
final static int TITLE_SIZE = 16;
final int FONT_COLOR = color(20);
final static String X_LABEL = "Like / Dislike Ratio";
final static String Y_LABEL = "Views";
final static String Z_LABEL = "Trending Date";
final static String API_KEY = "AIzaSyCimRo5OLoS0ezBaodtgx291IKR0tzRinY"; // change this to your own Youtube Data API key

PeasyCam cam;
ControlP5 cp5;

Table table;
int rows;
Random random = new Random();
List<Video> videos = new ArrayList();
int minViews = Integer.MAX_VALUE, maxViews = 0;
int minLikes = Integer.MAX_VALUE, maxLikes = 0;
int minDislikes = Integer.MAX_VALUE, maxDislikes = 0;
float minRatio = Float.MAX_VALUE, maxRatio = 0.0f;
int minDuration = Integer.MAX_VALUE, maxDuration = 0;
LocalDate earliestDate, latestDate;
int maxDays;
HashMap<String, List<Video>> videoMap = new HashMap();
float[] rotations = new float[3];
String currentId;
String currentName;
enum DisplayState {
    ALL, CURRENT
}
DisplayState displayState = DisplayState.ALL;
boolean toggle_1, toggle_2, toggle_10, toggle_15, toggle_17, toggle_20, toggle_22, toggle_23, toggle_24, toggle_25, toggle_26, toggle_27, toggle_28;
static String g_channelName, g_thumbnailLink;
static long g_viewCount, g_subscriberCount, g_videoCount;


public void setup() {
    
    textMode(SHAPE);
    loadData();
    println(earliestDate);
    setupCamera();
    setupCP5();
}

public void draw() {
    background(215);
    drawAxis();
    drawHUD();
    switch (displayState) {
        case ALL:
            for (Video video : videos) {
                video.render(NORMAL_ALPHA);
            }
            for (Map.Entry<String, List<Video>> vds : videoMap.entrySet()) {
                List<Video> videoList = vds.getValue();
                if (videoList.get(0).shouldSkip()) continue;
                beginShape();
                noFill();
                for (int i = 0; i < videoList.size(); i++) {
                    strokeWeight(6);
                    stroke(videoList.get(i).c, NORMAL_ALPHA);
                    curveVertex(videoList.get(i).x, videoList.get(i).y, videoList.get(i).z);
                }
                endShape();
            }
            break;
        case CURRENT:
            getAndDrawChannelInfo();
            for (Video video : videos) {
                if (mousePressed) {
                    if (mouseButton == RIGHT) {
                        displayState = DisplayState.ALL;
                    }
                }
                if (video.videoId.equals(currentId)) {
                    video.render(HIGH_ALPHA);
                } else {
                    video.render(LOW_ALPHA);
                }
            }
            for (Map.Entry<String, List<Video>> vds : videoMap.entrySet()) {
                List<Video> videoList = vds.getValue();
                if (videoList.get(0).shouldSkip()) continue;
                int alpha = LOW_ALPHA;
                if (videoList.get(0).videoId.equals(currentId)) {
                    alpha = HIGH_ALPHA;
                }
                if (mousePressed) {
                    if (mouseButton == RIGHT) {
                        displayState = DisplayState.ALL;
                    }
                }
                beginShape();
                noFill();
                for (int i = 0; i < videoList.size(); i++) {
                    strokeWeight(6);
                    stroke(videoList.get(i).c, alpha);
                    curveVertex(videoList.get(i).x, videoList.get(i).y, videoList.get(i).z);
                }
                endShape();
            }
            break;
    }
}
public void setupCamera() {
    cam = new PeasyCam(this, DIMENSION / 2, DIMENSION / 2, DIMENSION / 2, Z_DISTANCE);
    cam.setWheelScale(0.1f);
    float fov = PI / 3.0f;
    float cameraZ = (height / 2.0f) / tan(fov / 2.0f);
    perspective(fov, PApplet.parseFloat(width) / PApplet.parseFloat(height), 0.001f, cameraZ * 50.0f);
}
public void setupCP5() {
    cp5 = new ControlP5(this);
    int d = 30;
    addToggle("toggle_1", "Film & Animation", color(0xffa6cee3), 15, height - d * 13);
    addToggle("toggle_10", "Music", color(0xff1f78b4), 15, height - d * 12);
    addToggle("toggle_15", "Pets & Animals", color(0xffb2df8a), 15, height - d * 11);
    addToggle("toggle_17", "Sports", color(0xff33a02c), 15, height - d * 10);
    addToggle("toggle_20", "Gaming", color(0xfffb9a99), 15, height - d * 9);
    addToggle("toggle_22", "People & Blogs", color(0xffe31a1c), 15, height - d * 8);
    addToggle("toggle_23", "Comedy", color(0xfffdbf6f), 15, height - d * 7);
    addToggle("toggle_24", "Entertainment", color(0xffff7f00), 15, height - d * 6);
    addToggle("toggle_25", "News & Politics", color(0xffcab2d6), 15, height - d * 5);
    addToggle("toggle_26", "Howto & Style", color(0xff6a3d9a), 15, height - d * 4);
    addToggle("toggle_27", "Education", color(0xffffff99), 15, height - d * 3);
    addToggle("toggle_28", "Science & Technology", color(0xffb15928), 15, height - d * 2);
    addToggle("toggle_2", "Other", color(0xffffffff), 15, height - d * 1);
    cp5.setAutoDraw(false);
}

public void addToggle(String toggle, String label, int c, int x, int y) {
    cp5.addToggle(toggle)
        .setPosition(x, y)
        .setColorForeground(getSemi(c))
        .setColorBackground(getSemi(c))
        .setColorActive(c)
        .setColorLabel(FONT_COLOR)
        .setSize(15, 15)
        .setLabel(label)
        .setValue(true);
}
public void drawAxis() {
    pushMatrix();
//    translate(DIMENSION / 2, DIMENSION / 2, DIMENSION / 2);
    noFill();
    stroke(40, 20);
    strokeWeight(2);
//    box(DIMENSION);
    line(0, 0, 0, DIMENSION, 0, 0); // x
    line(0, 0, 0, 0, DIMENSION, 0); // y
    line(0, 0, 0, 0, 0, DIMENSION); // z
    popMatrix();

    // draw text label
    // x axis
    fill(FONT_COLOR);
    textSize(24);
    textAlign(RIGHT, BOTTOM);
    pushMatrix();
    translate(DIMENSION, -10, 0);
    text(X_LABEL, 0, 0);
    popMatrix();
    // y axis
    pushMatrix();
    translate(-10, DIMENSION, 0);
    rotate(3*HALF_PI, 0, 0, 1);
    text(Y_LABEL, 0, 0);
    popMatrix();

    // z axis
    textAlign(LEFT, BOTTOM);
    pushMatrix();
    translate(0, -10, DIMENSION);
    rotate(HALF_PI, 0, 1, 0);
    text(Z_LABEL, 0, 0);
    popMatrix();
}

public void drawHUD() {
    hint(DISABLE_DEPTH_TEST);
    cam.beginHUD();
    // draw title
    textSize(TITLE_SIZE);
    textAlign(LEFT, TOP);
    fill(FONT_COLOR);
    text("Trending YouTube Videos\n2017/11/14 - 2018/06/14", 15, 15);

    cp5.draw();
    cam.endHUD();
    hint(ENABLE_DEPTH_TEST);
}

public void getAndDrawChannelInfo() {
    if (g_channelName == null || !g_channelName.equals(currentName)) {
        getStats(currentName);
    }
    // draw channel info
    hint(DISABLE_DEPTH_TEST);
    cam.beginHUD();
    textSize(TITLE_SIZE);
    textAlign(RIGHT, TOP);
    fill(FONT_COLOR);
    int d = 25;
    text("Channel: " + g_channelName, width - 15, 15 + d * 0);
    text("View Count: " + g_viewCount, width - 15, 15 + d * 1);
    text("Subscriber Count: " + g_subscriberCount, width - 15, 15 + d * 2);
    text("Video Count: " + g_videoCount, width - 15, 15 + d * 3);
    PImage img = loadImage(g_thumbnailLink, "png");
    image(img, width - 15 - 240, 15 + d * 4);
    cp5.draw();
    cam.endHUD();
    hint(ENABLE_DEPTH_TEST);
}
public void loadData() {
    String filename = "USvideos.csv";
    table = loadTable(filename, "header");
    rows = table.getRowCount();
    for (int i = 0; i < rows; i++) {
        LocalDate date = convertToDate(table.getString(i, 1));
        if (i == 0) {
            earliestDate = date;
            latestDate = date;
        } else {
            earliestDate = earliestDate.isBefore(date) ? earliestDate : date;
            latestDate = latestDate.isAfter(date) ? latestDate : date;
        }
        String title = table.getString(i, 2);
        String channelTitle = table.getString(i, 3);
        int categoryId = table.getInt(i, 4);
        String publishTime = table.getString(i, 5);
        LocalDate publishDate = convertToDate2(publishTime.substring(0, 10));
        int duration = daysBetween(publishDate, date);
        minDuration = minDuration < duration ? minDuration : duration;
        maxDuration = maxDuration > duration ? maxDuration : duration;
        int views = table.getInt(i, 7); // most views = 225211923, less than int max
        maxViews = maxViews > views ? maxViews : views;
        minViews = minViews < views ? minViews : views;
        int likes = table.getInt(i, 8);
        maxLikes = maxLikes > likes ? maxLikes : likes;
        minLikes = minLikes < likes ? minLikes : likes;
        int dislikes = table.getInt(i, 9);
        maxDislikes = maxDislikes > dislikes ? maxDislikes : dislikes;
        minDislikes = minDislikes < dislikes ? minDislikes : dislikes;
        float ratio = likes / (float) dislikes;
        minRatio = minRatio < ratio ? minRatio : ratio;
        maxRatio = maxRatio > ratio ? maxRatio : ratio;
        String videoId = table.getString(i, 0);
        Video video = new Video(date, title, channelTitle, categoryId, views, likes, dislikes, videoId, duration);
        videos.add(video);
        if (!videoMap.containsKey(videoId)) {
            videoMap.put(videoId, new ArrayList());
        }
        videoMap.get(videoId).add(video);
    }
    maxDays = daysBetween(earliestDate, latestDate);
}

public LocalDate convertToDate(String str) {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yy.dd.MM", Locale.ENGLISH);
    LocalDate date = LocalDate.parse(str, formatter);
    return date;
}

public LocalDate convertToDate2(String str) {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd", Locale.ENGLISH);
    LocalDate date = LocalDate.parse(str, formatter);
    return date;
}

public int daysBetween(LocalDate d1, LocalDate d2) {
    return PApplet.parseInt(ChronoUnit.DAYS.between(d1, d2));
}

public int getSemi(int c) {
    return color(red(c), green(c), blue(c), alpha(c) / 2.0f);
}
class Video {
    LocalDate trendingDate;
    String title;
    String channelTitle;
    int categoryId;
    int views;
    int likes;
    int dislikes;
    float ratio;
    String videoId;
    int duration;
    float x, y, z;
    int c;

    Video(LocalDate td, String t, String ct, int ci, int v, int l, int d, String vi, int du) {
        this.trendingDate = td;
        this.title = t;
        this.channelTitle = ct;
        this.categoryId = ci;
        this.views = v;
        this.likes = l;
        this.dislikes = d;
        this.ratio = likes / (float) dislikes;
        this.videoId = vi;
        this.duration = du;
        setColor();
    }

    public void render(int alpha) {
        if (shouldSkip()) return;
        pushMatrix();
        this.x = constrain(map(this.ratio, minRatio, maxRatio, 0, DIMENSION), 0, DIMENSION);
        this.y = constrain(map(log(this.views), log(minViews), log(maxViews), 0, DIMENSION), 0, DIMENSION);
        this.z = constrain(map(daysBetween(earliestDate, this.trendingDate), 0, maxDays, 0, DIMENSION), 0, DIMENSION);
        translate(this.x, this.y, this.z);
        strokeWeight(2);
        stroke(this.c, alpha);
        point(0, 0, 0);
        if (displayState == DisplayState.ALL || displayState == DisplayState.CURRENT && this.videoId.equals(currentId)) {
            float mouseDist = sq(mouseX-screenX(0, 0, 0))+sq(mouseY-screenY(0, 0, 0));
            if (mouseDist < 20) {
                pushMatrix();
                rotations = cam.getRotations();
                rotateX(rotations[0]);
                rotateY(rotations[1]);
                rotateZ(rotations[2]);
                stroke(this.c, 255);
                strokeWeight(10);
                point(0,0,0);
                stroke(0,0,0,0);
                fill(255,255,255);
                textMode(SHAPE);
                textSize(24);
                fill(FONT_COLOR);
                text(this.title + "\nViews: " + String.valueOf(this.views) + " | Likes: " + String.valueOf(this.likes) + " | Dislikes: " + String.valueOf(this.dislikes) + "\nTrending Date: " + this.trendingDate.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")),0,0);
                try {
                    PImage img = loadImage("https://img.youtube.com/vi/"+this.videoId+"/hqdefault.jpg");
                    image(img, 0, 110);
                } catch (NullPointerException e) {}
                finally {
                    popMatrix();
                }
                if (mousePressed) {
                    if (mouseButton == LEFT) {
                        currentId = this.videoId;
                        currentName = this.channelTitle;
                        displayState = DisplayState.CURRENT;
                    }
                }
            }
        }
        popMatrix();
    }

    public void setColor() {
        switch (this.categoryId) {
            case 2:
            case 19:
            case 29:
            case 43:
                this.c = 0xffffffff;
                break;
            case 1:
                this.c = 0xffa6cee3;
                break;
            case 10:
                this.c = 0xff1f78b4;
                break;
            case 15:
                this.c = 0xffb2df8a;
                break;
            case 17:
                this.c = 0xff33a02c;
                break;
            case 20:
                this.c = 0xfffb9a99;
                break;
            case 22:
                this.c = 0xffe31a1c;
                break;
            case 23:
                this.c = 0xfffdbf6f;
                break;
            case 24:
                this.c = 0xffff7f00;
                break;
            case 25:
                this.c = 0xffcab2d6;
                break;
            case 26:
                this.c = 0xff6a3d9a;
                break;
            case 27:
                this.c = 0xffffff99;
                break;
            case 28:
                this.c = 0xffb15928;
                break;
        }
    }

    public boolean shouldSkip() {
        if (this.categoryId == 1 && !toggle_1) return true;
        else if (this.categoryId == 2 || this.categoryId == 19 || this.categoryId == 29 || this.categoryId == 43 && !toggle_2) return true;
        else if (this.categoryId == 10 && !toggle_10) return true;
        else if (this.categoryId == 15 && !toggle_15) return true;
        else if (this.categoryId == 17 && !toggle_17) return true;
        else if (this.categoryId == 20 && !toggle_20) return true;
        else if (this.categoryId == 22 && !toggle_22) return true;
        else if (this.categoryId == 23 && !toggle_23) return true;
        else if (this.categoryId == 24 && !toggle_24) return true;
        else if (this.categoryId == 25 && !toggle_25) return true;
        else if (this.categoryId == 26 && !toggle_26) return true;
        else if (this.categoryId == 27 && !toggle_27) return true;
        else if (this.categoryId == 28 && !toggle_28) return true;
        else return false;
    }
}
public void getStats(String channelName) {
    try {
        GetRequest get = new GetRequest("https://www.googleapis.com/youtube/v3/channels?part=snippet,contentDetails,statistics&forUsername=" + URLEncoder.encode(channelName, "UTF-8") + "&key=" + API_KEY);
        get.send();
        org.json.JSONObject result = new JSONObject(get.getContent());
        g_channelName = channelName;
        setGlobals(result);
    } catch (Exception e) {
        e.printStackTrace();
    }
}

public void setGlobals(org.json.JSONObject obj) {
    int totalResults = obj.getJSONObject("pageInfo").getInt("totalResults");
    if (totalResults == 0) {
        g_viewCount = 0;
        g_subscriberCount = 0;
        g_videoCount = 0;
        g_thumbnailLink = "https://yt3.ggpht.com/a-/AAuE7mB9WSnjk6VfFfP1k2uSFfBpgr5pejFcB6CDvw=s240-mo-c-c0xffffffff-rj-k-no";
    } else {
        org.json.JSONObject item = obj.getJSONArray("items").getJSONObject(0);
        org.json.JSONObject statistics = item.getJSONObject("statistics");
        g_viewCount = Long.parseLong(statistics.getString("viewCount"));
        g_subscriberCount = Long.parseLong(statistics.getString("subscriberCount"));
        g_videoCount = Long.parseLong(statistics.getString("videoCount"));
        org.json.JSONObject thumbnails = item.getJSONObject("snippet").getJSONObject("thumbnails");
        g_thumbnailLink = thumbnails.getJSONObject("medium").getString("url");
    }
}


  public void settings() {  fullScreen(P3D); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "trending_video" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
