/********************************************************
 * Data Visualization - Most popular items by year      *
 * Tested in Processing 3.5.3                           *
 *                                                      *
 * Author: Jiaheng Tang                                 *
 * Supervisor: George Legrady                           *
 * Adapted from Mert Toka's work in MAT 259 in 2016     *
 *                                                      *
 * Purpose: Visualizing the trending Youtube videos     *
 *          from 2017/11/14 to 2018/06/14               *
 *                                                      *
 * Usage:   Press toggle button on bottom left side     *
 *          to toggle records by category.              *
 *          Hover over records to show their title      *
 *          likes, dislikes, and views.                 *
 *          Click on the records to show their channel  *
 *          info. Right-click to cancel highlight       *
 *                                                      *
 ********************************************************/

import java.util.*;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import peasy.*;
import controlP5.*;
import http.requests.*;
import java.net.URLEncoder;
import org.json.*;
import org.json.JSONObject;
import org.json.JSONArray;

final static int Z_DISTANCE = 2200;
final static float DIMENSION = 2000;
final static int HIGH_ALPHA = 255;
final static int NORMAL_ALPHA = 100;
final static int LOW_ALPHA = 10;
final static int TITLE_SIZE = 16;
final color FONT_COLOR = color(20);
final static String X_LABEL = "Like / Dislike Ratio";
final static String Y_LABEL = "Views";
final static String Z_LABEL = "Trending Date";
final static String API_KEY = "AIzaSyCimRo5OLoS0ezBaodtgx291IKR0tzRinY"; // change this to your own Youtube Data API key

PeasyCam cam;
ControlP5 cp5;

Table table;
int rows;
Random random = new Random();
List<Video> videos = new ArrayList();
int minViews = Integer.MAX_VALUE, maxViews = 0;
int minLikes = Integer.MAX_VALUE, maxLikes = 0;
int minDislikes = Integer.MAX_VALUE, maxDislikes = 0;
float minRatio = Float.MAX_VALUE, maxRatio = 0.0;
int minDuration = Integer.MAX_VALUE, maxDuration = 0;
LocalDate earliestDate, latestDate;
int maxDays;
HashMap<String, List<Video>> videoMap = new HashMap();
float[] rotations = new float[3];
String currentId;
String currentName;
enum DisplayState {
    ALL, CURRENT
}
DisplayState displayState = DisplayState.ALL;
boolean toggle_1, toggle_2, toggle_10, toggle_15, toggle_17, toggle_20, toggle_22, toggle_23, toggle_24, toggle_25, toggle_26, toggle_27, toggle_28;
static String g_channelName, g_thumbnailLink;
static long g_viewCount, g_subscriberCount, g_videoCount;


void setup() {
    fullScreen(P3D);
    textMode(SHAPE);
    loadData();
    println(earliestDate);
    setupCamera();
    setupCP5();
}

void draw() {
    background(215);
    drawAxis();
    drawHUD();
    switch (displayState) {
        case ALL:
            for (Video video : videos) {
                video.render(NORMAL_ALPHA);
            }
            for (Map.Entry<String, List<Video>> vds : videoMap.entrySet()) {
                List<Video> videoList = vds.getValue();
                if (videoList.get(0).shouldSkip()) continue;
                beginShape();
                noFill();
                for (int i = 0; i < videoList.size(); i++) {
                    strokeWeight(6);
                    stroke(videoList.get(i).c, NORMAL_ALPHA);
                    curveVertex(videoList.get(i).x, videoList.get(i).y, videoList.get(i).z);
                }
                endShape();
            }
            break;
        case CURRENT:
            getAndDrawChannelInfo();
            for (Video video : videos) {
                if (mousePressed) {
                    if (mouseButton == RIGHT) {
                        displayState = DisplayState.ALL;
                    }
                }
                if (video.videoId.equals(currentId)) {
                    video.render(HIGH_ALPHA);
                } else {
                    video.render(LOW_ALPHA);
                }
            }
            for (Map.Entry<String, List<Video>> vds : videoMap.entrySet()) {
                List<Video> videoList = vds.getValue();
                if (videoList.get(0).shouldSkip()) continue;
                int alpha = LOW_ALPHA;
                if (videoList.get(0).videoId.equals(currentId)) {
                    alpha = HIGH_ALPHA;
                }
                if (mousePressed) {
                    if (mouseButton == RIGHT) {
                        displayState = DisplayState.ALL;
                    }
                }
                beginShape();
                noFill();
                for (int i = 0; i < videoList.size(); i++) {
                    strokeWeight(6);
                    stroke(videoList.get(i).c, alpha);
                    curveVertex(videoList.get(i).x, videoList.get(i).y, videoList.get(i).z);
                }
                endShape();
            }
            break;
    }
}
