void loadData() {
    String filename = "USvideos.csv";
    table = loadTable(filename, "header");
    rows = table.getRowCount();
    for (int i = 0; i < rows; i++) {
        LocalDate date = convertToDate(table.getString(i, 1));
        if (i == 0) {
            earliestDate = date;
            latestDate = date;
        } else {
            earliestDate = earliestDate.isBefore(date) ? earliestDate : date;
            latestDate = latestDate.isAfter(date) ? latestDate : date;
        }
        String title = table.getString(i, 2);
        String channelTitle = table.getString(i, 3);
        int categoryId = table.getInt(i, 4);
        String publishTime = table.getString(i, 5);
        LocalDate publishDate = convertToDate2(publishTime.substring(0, 10));
        int duration = daysBetween(publishDate, date);
        minDuration = minDuration < duration ? minDuration : duration;
        maxDuration = maxDuration > duration ? maxDuration : duration;
        int views = table.getInt(i, 7); // most views = 225211923, less than int max
        maxViews = maxViews > views ? maxViews : views;
        minViews = minViews < views ? minViews : views;
        int likes = table.getInt(i, 8);
        maxLikes = maxLikes > likes ? maxLikes : likes;
        minLikes = minLikes < likes ? minLikes : likes;
        int dislikes = table.getInt(i, 9);
        maxDislikes = maxDislikes > dislikes ? maxDislikes : dislikes;
        minDislikes = minDislikes < dislikes ? minDislikes : dislikes;
        float ratio = likes / (float) dislikes;
        minRatio = minRatio < ratio ? minRatio : ratio;
        maxRatio = maxRatio > ratio ? maxRatio : ratio;
        String videoId = table.getString(i, 0);
        Video video = new Video(date, title, channelTitle, categoryId, views, likes, dislikes, videoId, duration);
        videos.add(video);
        if (!videoMap.containsKey(videoId)) {
            videoMap.put(videoId, new ArrayList());
        }
        videoMap.get(videoId).add(video);
    }
    maxDays = daysBetween(earliestDate, latestDate);
}

LocalDate convertToDate(String str) {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yy.dd.MM", Locale.ENGLISH);
    LocalDate date = LocalDate.parse(str, formatter);
    return date;
}

LocalDate convertToDate2(String str) {
    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd", Locale.ENGLISH);
    LocalDate date = LocalDate.parse(str, formatter);
    return date;
}

int daysBetween(LocalDate d1, LocalDate d2) {
    return int(ChronoUnit.DAYS.between(d1, d2));
}

color getSemi(color c) {
    return color(red(c), green(c), blue(c), alpha(c) / 2.0);
}
